#include "tests.h"
#include "distributions.h"


#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

#include <cstdlib>
#include <ctime>

long double adcval;

long double kscval;

long double chisqcval;

inline long double pnorm(long double x){
  return erfc((-x)/sqrt(2))/2;
}

extern unsigned int jackknife;
extern unsigned int sample_bias_step_ncut;
extern long double *sample_bias_step_cuts;
extern long double *sample_bias_step_cum_probs;


using namespace std;

void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to);

//Anderson-Darling Test

void  subsample(unsigned int n,unsigned int x,long double *data,long double *to){
  unsigned int rmx=n*(RAND_MAX/n);
  for(unsigned int i=0;i<x;i++){
    unsigned int rnd=rand();
    while(rnd>rmx){
      rnd=rand(); //redraw if too large.
    };
    *(to+i)=*(data+rnd%n);
  };  
};


void sortedsubsample(unsigned int n,unsigned int x,unsigned int *to){
  //Gives frequencies of each observation.
  for(unsigned int i=0;i<n;i++){
    *(to+i)=0;
  };
  unsigned int rmx=n*(RAND_MAX/n);
  for(unsigned int i=0;i<x;i++){
    unsigned int rnd=rand();
    while(rnd>rmx){
      rnd=rand(); //redraw if too large.
    };
    (*(to+rnd%n))++;
  };  
};

  boost::mt19937 rng;                 // produces randomness out of thin air
                                    // see pseudo-random number generators

unsigned int rbinom(unsigned int n,long double p){
  unsigned int cuttoff=p*RAND_MAX;
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    if((unsigned int)rand()<cuttoff){
      ans++;
    };
  };
  return ans;
};


void makebintable(unsigned int n,long double p,unsigned int min,unsigned int max,long double* to){
  //  cout<<"n="<<n<<", p="<<p<<", min="<<min<<", max="<<max<<"\n";
  long double prob=1;
  *to=1;
  long double a=-p/(1-p);
  long double b=-a*(n+1);
  //  cout<<"a="<<a<<", b="<<b<<"\n";
  for(unsigned int i=min;i<max;i++){
    prob*=a+b/(i+1);
    //    cout<<prob<<" ";
    *(to+i-min+1)=*(to+i-min)+prob;
  };
  for(unsigned int i=0;i<=max-min;i++){
    *(to+i)/=*(to+max-min);
    //        cout<<*(to+i)<<" ";
  };  
  //    cout<<"\n";
};


unsigned int getrndfromlkup(unsigned int min,unsigned int max,const long double *lkup){
  unsigned int mn=0;
  unsigned int mx=max-min;
  long double v=((long double)rand())/RAND_MAX;
  //  cout<<v<<"\n";
  while(mx-mn>1){
    unsigned int test=(mx+mn)/2;
    //      cout<<test<<" "<<*(lkup+test)<<"\n";
    if(v<*(lkup+test)){
      mx=test;
    }else{
      mn=test;
    };    
  };
  if(v<(*(lkup+mn))){
      return min+mn;
    }else{
      return min+mx;
    };
};

unsigned int rbinom_fast(unsigned int n,long double p){
  if(n<400){
    return rbinom(n,p);
  };


  //  cout<<"\n";
  unsigned int N=pow(n,0.666666666666666666667);

  long double mean=N*p;
  long double sd=sqrt(N*p*(1-p));
  long double minplaus=mean-8*sd;
  unsigned int min=minplaus;
  if(minplaus<0){
    min=0;
  };
  long double maxplaus=mean+8*sd;
  unsigned int max=maxplaus;
  if(maxplaus<0){
    max=0;
  };
  long double *lkuptab=new long double[max+1-min];
  makebintable(N,p,min,max,lkuptab);
  //Start by making a lookuptable for size N. Then generate n/N random
  //binomials for N and sum them.

  unsigned int k=n/N;
  unsigned int tot=0;
  //  cout<<"N="<<N<<", k="<<k<<"\n"; 
  for(unsigned int i=0;i<k;i++){
    unsigned int rbinN=getrndfromlkup(min,max,lkuptab);
    tot+=rbinN;
    //    cout<<rbinN<<"  ";
  };
  //  cout<<"\n";
  unsigned int rem=n-k*N;
  delete[] lkuptab;
  //  cout<<"N="<<N<<", k="<<k<<", rem="<<rem<<"\n";
  return tot+rbinom_fast(rem,p);
};



unsigned int rhgeom(unsigned int n,unsigned int success,unsigned int tot){
  unsigned int ans=0;
  for(unsigned int i=0;i<n;i++){
    unsigned int cuttoff=(((long unsigned int)success)*RAND_MAX)/tot;
    if((unsigned int)rand()<cuttoff){
      ans++;
      success--;
    };
    tot--;
  };
  return ans;
};


unsigned int rbinom_boost(unsigned int n,long double p){
  //Transformed rejection method


  boost::binomial_distribution<> my_binomial(n,p);      // binomial distribution with n=20, p=0.5
                                         // see random number distributions
 boost::variate_generator<boost::mt19937&, boost::binomial_distribution<> > next_value(rng, my_binomial);     // glues randomness with mapping
 return next_value();

//
//  if(p>0.5){
//    return n-rbinom(n,1-p);
//  };
//  if(n*p>=10){
//    long double u=((long double) rand())/RAND_MAX-0.5;
//    long double us=0.5-abs(u);
//    long double v=((long double) rand())/RAND_MAX;  
//    long double b=1.15+2.53*sqrt(n*p*(1-p));
//    long double a=0.01*p+0.0248*b-0.0873;
//    long double c=n*p+0.5;
//    long double vr=0.92-4.2/b;
//    int k=(2*a/us+b)*u+c;
//    
//    if(us>0.07&&v<=vr){
//      while(k<0 || k>n){
//	long double u=((long double) rand())/RAND_MAX-0.5;
//	long double us=0.5-abs(u);
//	k=(2*a/us+b)*u+c;
//      };   
//      return k;
//    };
//
//    long double alpha=(2.83+5.1/b)*sqrt(n*p*(1-p));
//    long double lpq=
//
//  }else{
//
//  };
};

void sortedsubsample(unsigned int n,const unsigned int *counts,unsigned int x,unsigned int *to){
  //Gives frequencies of each observation.
  if(x<n){
    unsigned int *cumcount=new unsigned int[n];
    for(unsigned int i=0;i<n;i++){
      *(to+i)=0;
      if(i==0){
	*cumcount=*counts;
      }else{
	*(cumcount+i)=*(cumcount+i-1)+*(counts+i);
      };
    };
    unsigned int tc=*(cumcount+n-1);
    unsigned int rmx=tc*(RAND_MAX/tc);
    for(unsigned int i=0;i<x;i++){
      unsigned int rnd=rand();
      while(rnd>rmx){
	rnd=rand(); //redraw if too large.
      };
      rnd=rnd%tc;
      if(rnd<*cumcount){
	(*to)++;
      }else{
	unsigned int upper=n;
	unsigned int lower=0;
	while(upper-lower>1){
	  unsigned int mid=(upper+lower)/2;
	  if(*(cumcount+mid)<=rnd){
	    lower=mid;
	  }else{
	    upper=mid;
	  };
	};
	(*(to+upper))++;
      };
    };  
    delete[] cumcount;
  }else{
    unsigned int *cumcount=new unsigned int[n];
    *(cumcount+n-1)=*(counts+n-1);
    for(unsigned int i=n-1;i>0;i--){
      *(cumcount+i-1)=*(cumcount+i)+*(counts+i-1);
    };    
    unsigned int tc=*(cumcount);
    //    unsigned int rmx=tc*(RAND_MAX/tc);
    unsigned int rc=x;
    for(unsigned int i=0;i<n;i++){
      *(to+i)=rbinom_fast(rc,*(counts+i)/((long double)*(cumcount+i)));
      rc-=*(to+i);
    };
    delete[] cumcount;
  };
};

long double CalculateADstat(unsigned int n,const long double *data, const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data);
    //calculate adstat
  long double ADstat=-1;
  unsigned int Fn=0;
  long double Fsp=0;
  long double Fsn=0;
  //    long double Fsnc=0;
  for(unsigned int j=0;j<n;j++){
    Fsp=tempdist->dist(data[j]);
    Fsn=1-Fsp;
//      Fsnc=1-pnorm(data[199-j]);
//      //Should be complementary, but this avoids rounding errors
//      Fn++;
//      ADstat-=(2*(long double)Fn-1)/40000*(log(Fsnc)+log(Fsp));
    
    ADstat-=(1-((long double)Fn)/n)*(1-((long double)Fn)/n)*log(Fsn);
    ADstat+=(((long double)Fn)/n)*(((long double)Fn)/n)*log(Fsp);
    Fn++;
    ADstat+=(1-((long double)Fn)/n)*(1-((long double)Fn)/n)*log(Fsn);
    ADstat-=(((long double)Fn)/n)*(((long double)Fn)/n)*log(Fsp);
  };
  long double ans=ADstat*n*tempdist->adjustADstat(n);
  delete tempdist;
  return ans;
};

long double CalculateADstat(unsigned int x,unsigned int n,const long double *data, const Distribution &dist,const unsigned int *samp){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data,samp);
  long double ADstat=-1;
  unsigned int Fn=0;
  long double Fsp=0;
  long double Fsn=0;
  for(unsigned int i=0;i<n;i++){
    if(*(samp+i)){
      Fsp=tempdist->dist(*(data+i));
      Fsn=tempdist->survive(*(data+i));
      //Should be complementary, but this avoids rounding errors
      //if distribution function is close to 1, rounding errors can be fatal. 
      ADstat-=(1-((long double)Fn)/x)*(1-((long double)Fn)/x)*log(Fsn);
      ADstat+=(((long double)Fn)/x)*(((long double)Fn)/x)*log(Fsp);
      Fn+=*(samp+i);
      ADstat+=(1-((long double)Fn)/x)*(1-((long double)Fn)/x)*log(Fsn);
      ADstat-=(((long double)Fn)/x)*(((long double)Fn)/x)*log(Fsp);
    };
  };
  delete[] samp;
  ADstat*=x* tempdist->adjustADstat(x);
  if(isnan(ADstat)){
    cout<<"NAN found as AD statistic. Changing to 1000.\n";
    ADstat=1000;  //This will normally lead to rejecting the test.
  };
  delete tempdist;
  return ADstat;
};

char ADtest(unsigned int x,const void *data,const Distribution &dist){
  //Anderson-Darling Test
  //Assumes data are sorted
  Data *A=(Data *)data;
  unsigned int *samp=new unsigned int[A->n];
  
  sortedsubsample(A->n,x,samp);
  
  long double ADstat=CalculateADstat(x,A->n,A->data,dist,samp);
  return(ADstat>adcval);
};

inline unsigned int randfrom(unsigned int x){
  //generates a random number from 0 to x-1
  unsigned int rm=x*(RAND_MAX/x)-1;
  unsigned int ans=rand();
  for(;ans>rm;ans=rand());
  return ans%x;
};

inline unsigned int randnumless(unsigned int x,unsigned int l,unsigned int u){
  //generates a random number indicating the number of samples from a
  //subsample of size x from a sample of size l+u that fall in a given
  //subset of size l.

  //Should probably make this more efficient.
  unsigned int ans=0;
  for(unsigned int i=0;i<x;i++){
    unsigned int rnd=randfrom(l+u);
    if(rnd<l){
      l--;
      ans++;
    }else{
      u--;
    };
  };
  return ans;
};


void   sortedjackknife(unsigned int n,unsigned int x,const long double *data,long double *to){
  //Takes a subsample of data of size x without replacement. 
  //Writes the result into to.
  unsigned int pos=randfrom(n);
  unsigned int nl=randnumless(x-1,pos,n-pos-1);
  *(to+nl)=*(data+pos);
  if(nl>0){
    sortedjackknife(pos,nl,data,to);
  };
  if(nl<x-1){
    sortedjackknife(n-pos-1,x-nl-1,data+pos+1,to+nl+1);    
  };
};




void   sortedjackknife(unsigned int n,const unsigned int *counts,unsigned int x,unsigned int *to){
  //Takes a subsample of grouped data of size x without replacement. 
  //Writes the result into to.
    unsigned int *cumcount=new unsigned int[n];
    *(cumcount+n-1)=*(counts+n-1);
    for(unsigned int i=n-1;i>0;i--){
      *(cumcount+i-1)=*(cumcount+i)+*(counts+i-1);
    };    
    unsigned int tc=*(cumcount);
    //    unsigned int rmx=tc*(RAND_MAX/tc);
    unsigned int rc=x;
    for(unsigned int i=0;i<n;i++){
      *(to+i)=rhgeom(rc,*(counts+i),*(cumcount+i));
      rc-=*(to+i);
    };
    delete[] cumcount;
};



char ADtest_jackknife(unsigned int x,const void *data,const Distribution &dist){
  //Anderson-Darling Test
  //Assumes data are sorted
  Data *A=(Data *)data;
  long double *samp=new long double[x];

  sortedjackknife(A->n,x,A->data,samp);


  long double ADstat=CalculateADstat(x,samp,dist);
  return(ADstat>adcval);
};



//Kolmogorov-Smirnov test


long double CalculateKSstat(unsigned int n,const long double *data, const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data);
  long double KSstat=0;
  unsigned int Fn=0;
  long double Fsp=0;
  for(unsigned int i=0;i<n;i++){
    Fsp=n*tempdist->dist(*(data+i));
    if(Fsp-Fn>KSstat){
      KSstat=Fsp-Fn;
    };
    Fn++;
    if(Fn-Fsp>KSstat){
      KSstat=Fn-Fsp;
    };
  };
  delete tempdist;
  return KSstat/sqrt(n);
};

long double CalculateKSstat(unsigned int x,unsigned int n,const long double *data, const Distribution &dist,const unsigned int *samp){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data,samp);
  long double KSstat=0;
  unsigned int Fn=0;
  long double Fsp=0;
  for(unsigned int i=0;i<n;i++){
    if(*(samp+i)){
      Fsp=x*tempdist->dist(*(data+i));
      if(Fsp-Fn>KSstat){
	KSstat=Fsp-Fn;
      };
      Fn+=*(samp+i);
      if(Fn-Fsp>KSstat){
	KSstat=Fn-Fsp;
      };
    };
  };
  delete tempdist;
  return KSstat/sqrt(x);
};


char KStest(unsigned int x,const void *data,const Distribution &dist){
  //Kolmogorov-Smirnov test
  //Assumes data are sorted
  Data *A=(Data *)data;
  unsigned int *samp=new unsigned int[A->n];

  sortedsubsample(A->n,x,samp);
  long double KSstat=CalculateKSstat(x,A->n,A->data,dist,samp);
  delete[] samp;
  return(KSstat>kscval);
};


char KStest_jackknife(unsigned int x,const void *data,const Distribution &dist){
  //Kolmogorov-Smirnov test
  //Assumes data are sorted
  Data *A=(Data *)data;
  long double *samp=new long double[x];

  sortedjackknife(A->n,x,A->data,samp);
  long double KSstat=CalculateKSstat(x,samp,dist);
  delete[] samp;
  return(KSstat>kscval);
};


long double CalculateChiSquarestat(unsigned int x,unsigned int n,const long double *data,const Distribution &dist,unsigned int *samp){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data,samp);
  long double ans=0;
  long double ldist=0;
  
  for(unsigned int i=0;i<n;i++){
    unsigned int bincount=0;
    for(;i<n&&(tempdist->dist(*(data+i))-ldist)*x<20;i++){
      bincount+=*(samp+i);
    };
    long double dst=1;
    if(i<n-1){
      dst=tempdist->dist(*(data+i));
    };
    ans+=(bincount-(dst-ldist)*x)*(bincount-(dst-ldist)*x)/((dst-ldist)*x);
    ldist=dst;
  };
  delete tempdist;
  return ans;  
};


long double CalculateChiSqstat(unsigned int n,const long double *data,const Distribution &dist){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,data);
  long double ans=0;
  long double ldist=0;
  
  for(unsigned int i=0;i<n;i++){
    unsigned int bincount=0;
    for(;i<n&&(tempdist->dist(*(data+i))-ldist)*n<20;i++){
      bincount++;
    };    
    long double dst=1;
    if(i<n-1){
      dst=tempdist->dist(*(data+i));
    };
    ans+=(bincount-(dst-ldist)*n)*(bincount-(dst-ldist)*n)/((dst-ldist)*n);
    ldist=dst;
  };
  delete tempdist;
  return ans;    
};

long double CalculateChiSquarestatGroup(unsigned int n,const long double *data,const Distribution &dist,unsigned int *counts){
  Distribution *tempdist=dist.copy();
  tempdist->estimate_data(n,counts,data);
  long double ans=0;
  long double ldist=0;
  
  unsigned int tot=0;
  for(unsigned int i=0;i<n;i++){
    tot+=*(counts+i);
  };

  for(unsigned int i=0;i<n;i++){
    long double dst=1;
    if(i<n-1){
      dst=tempdist->dist(*(data+i));
    };
    ans+=(*(counts+i)-(dst-ldist)*tot)*(*(counts+i)-(dst-ldist)*tot)/((dst-ldist)*tot);
    ldist=dst;
  };
  delete tempdist;
  return ans;    
};



char ChiSqtest(unsigned int x,const void *data,const Distribution &dist){
  //Chi squared test for count data.
  FreqTable *A=(FreqTable *)data;

  unsigned int *samp=new unsigned int[A->n];
  sortedsubsample(A->n,A->count,x,samp);
  long double chisq=CalculateChiSquarestat(x,A->n,A->data,dist,samp);
  delete[] samp;
  return(chisq>chisqcval);
};

char ChiSqtestGrouped(unsigned int x,const void *data,const Distribution &dist){
  //Chi squared test for count data.
  GroupedData *A=(GroupedData *)data;

  unsigned int *samp=new unsigned int[A->n];
  sortedsubsample(A->n,A->count,x,samp);
  long double chisq=CalculateChiSquarestatGroup(A->n,A->data,dist,samp);
  delete[] samp;
  return(chisq>chisqcval);
};


long double findmle_sample_bias_normal(unsigned int n,long double *samp,unsigned int ncut,long double *cutoffs,long double *nn=NULL){
  //Only works for normal distribution
  //n is sample size
  //samp is data points
  //ncut is the number of intervals
  //cutoffs gives the cutoffs between intervals
  //nn is a vector of proportions of each interval in the data. 
  //If it is set to NULL, then it is calculated in this function. However, 
  //if it is available e.g. from the simulation, then passing it to this 
  //function can speed up computation.
  unsigned int delnn=0;
  if(nn==NULL){
    delnn=1;
    nn=new long double[ncut+1];
    for(unsigned int i=0;i<ncut+1;i++){
      *(nn+i)=0;
    };
    for(unsigned int i=0;i<n;i++){
      //    cout<<*(samp+i)<<" ";
      unsigned int j=0;
      for(;(j<ncut)&&*(cutoffs+j)<*(samp+i);j++);
      //    cout<<j<<"\n";
      (*(nn+j))++;
    };
    for(unsigned int i=0;i<ncut+1;i++){
      *(nn+i)/=n;
    //    cout<<*(nn+i)<<" ";
    };
  };
  //  cout<<"\n";
  long double s=0;
  for(unsigned int i=0;i<n;i++){
    s+=*(samp+i)*(*(samp+i));
  };
  s/=n;
  long double dll=1;
  long double sigma=1;//sqrt(s);
  long double *pnc=new long double[ncut+1];
  long double *T=new long double[ncut+1];
  long double *ndph=new long double[ncut+2];
  long double *ddph=new long double[ncut+2];
  long double lastdll=1000000000;
  long double factor=1;
  unsigned int numstep=0;
  while(((dll>0.0000000001)||(dll<-0.0000000001))&&numstep<100){
    numstep++;
    *pnc=0;
    for(unsigned int i=0;i<ncut;i++){
      *(pnc+i+1)=pnorm(*(cutoffs+i)/sigma);
      *(T+i)=*(pnc+i+1)-*(pnc+i);
    };
    *(T+ncut)=1-*(pnc+ncut);
    dll=s/(sigma*sigma*sigma)-1/sigma;
    long double ddll=1/(sigma*sigma)-3*s/(sigma*sigma*sigma*sigma);
    for(unsigned int i=1;i<ncut+1;i++){
      *(ndph+i)=*(cutoffs+i-1)/(sigma*sigma)*exp(-(*(cutoffs+i-1))*(*(cutoffs+i-1))/(sigma*sigma)/2)/sqrt(2*boost::math::constants::pi<long double>());
      *(ddph+i)=(*(ndph+i))*(-2+(*(cutoffs+i-1))*(*(cutoffs+i-1))/(sigma*sigma))/sigma;
    };
    *(ndph+ncut+1)=0;
    *(ddph+ncut+1)=0;
    *(ndph)=0;
    *(ddph)=0;
    for(unsigned int i=0;i<ncut+1;i++){
      long double dT=*(ndph+i)-*(ndph+i+1);
      long double ddT=*(ddph+i+1)-*(ddph+i);
      dll-=*(nn+i)*dT/(*(T+i));
      ddll+=(*(nn+i))*(dT*dT/(*(T+i)*(*(T+i)))-(ddT/(*(T+i))));
    };

    if(ddll>0){
      ddll*=-2;
    };
    if(abs(dll)>abs(lastdll)){
      factor*=0.8;
    }else{
      if(factor/0.9<1){
	factor/=0.9;
      }else{
	factor=1;
      };
    };
    if(dll/ddll*factor>sigma){
      sigma/=2;
    }else{
      sigma=sigma-dll/ddll*factor;
    };
    lastdll=dll;
  };


  if(((dll>0.0000000001)||(dll<-0.0000000001))){
    //Failed to converge. Revert to interval bisection.
    long double stop=4;
    long double sbot=0;
    for(unsigned int i=0;i<100;i++){
      sigma=(stop+sbot)/2;
      *pnc=0;
      for(unsigned int i=0;i<ncut;i++){
	*(pnc+i+1)=pnorm(*(cutoffs+i)/sigma);
	*(T+i)=*(pnc+i+1)-*(pnc+i);
      };
      *(T+ncut)=1-*(pnc+ncut);
      dll=s/(sigma*sigma*sigma)-1/sigma;
      for(unsigned int i=1;i<ncut+1;i++){
	*(ndph+i)=*(cutoffs+i-1)/(sigma*sigma)*exp(-(*(cutoffs+i-1))*(*(cutoffs+i-1))/(sigma*sigma)/2)/sqrt(2*boost::math::constants::pi<long double>());
      };
      *(ndph+ncut+1)=0;
      *(ndph)=0;
      for(unsigned int i=0;i<ncut+1;i++){
	long double dT=*(ndph+i)-*(ndph+i+1);
	dll-=*(nn+i)*dT/(*(T+i));
      };	
      if(dll<0){
	stop=sigma;
      }else{
	sbot=sigma;
      };
    };
  };

  //            cout<<sigma<<" "<<dll<<" "<<ddll<<"\n";
  if(delnn){
    delete[] nn;
  };
  delete[] ndph;
  delete[] ddph;
  delete[] pnc;
  delete[] T;

  return(sigma);
};

char LRtestSampleBiasNormal(unsigned int x,const void *data,const Distribution &dist){
  //Likelihood ratio test against sampling bias. Assumes that distribution is normal.
  //Assumes data are sorted
  
  if(strcmp(dist.distname(),"normal")){
    cout<<"Likelihood ratio test not available for distribution "<<dist.distname()<<".\n";
    exit(0);
  }
  Data *A=(Data *)data;
  long double *samp=new long double[x];

  subsample(A->n,x,A->data,samp);

  long double *pnc=new long double[sample_bias_step_ncut+1];
  long double *T=  new long double[sample_bias_step_ncut+1];
  long double *p=  new long double[sample_bias_step_ncut+1];
  long double *nn=new long double[sample_bias_step_ncut+1];
  for(unsigned int i=0;i<sample_bias_step_ncut+1;i++){
    *(nn+i)=0;
  };
  for(unsigned int i=0;i<x;i++){
    unsigned int j=0;
    for(;(j<sample_bias_step_ncut)&&*(sample_bias_step_cuts+j)<*(samp+i);j++);
    (*(nn+j))++;
  };
  for(unsigned int i=0;i<sample_bias_step_ncut+1;i++){
    *(nn+i)/=x;
  };

  long double sigma=findmle_sample_bias_normal(x,samp,sample_bias_step_ncut,sample_bias_step_cuts,nn);

  *pnc=0;
  long double sum1=0;
  //  long double sumsig1=0;
  for(unsigned int i=0;i<sample_bias_step_ncut;i++){
    *(pnc+i+1)=pnorm(*(sample_bias_step_cuts+i)/sigma);
    *(T+i)=*(pnc+i+1)-*(pnc+i);
    *(p+i)=*(nn+i)/(*(T+i));
    if(*(nn+i)>0){
      sum1+=*(nn+i)*log(*(p+i));
      //      sumsig1+=*(nn+i)*log(*(nn+i)*sample_bias_step_ncut);    
    };
  };
  *(T+sample_bias_step_ncut)=1-*(pnc+sample_bias_step_ncut);
  *(p+sample_bias_step_ncut)=*(nn+sample_bias_step_ncut)/(*(T+sample_bias_step_ncut));
  if(*(nn+sample_bias_step_ncut)>0){
    sum1+=*(nn+sample_bias_step_ncut)*log(*(p+sample_bias_step_ncut));
    //    sumsig1+=*(nn+sample_bias_step_ncut)*log(*(nn+sample_bias_step_ncut)*sample_bias_step_ncut);    
  };
  long double s=0;
  for(unsigned int i=0;i<x;i++){
    s+=*(samp+i)*(*(samp+i));    
  };
  s/=x;
  long double ll= -s/sigma/sigma/2-log(sigma)+sum1;
  long double llmod= -(log(s)+1)/2;
  long double lrat=2*x*(ll-llmod);

  long double sigma2=1;
  long double p1=1;
  long double p2=1;
  long double p3=1;
  long double p4=1;
  long double p5=1;

  long double lltrue=-s/sigma2/sigma2/2-log(sigma2)+*nn*log(p1)+*(nn+1)*log(p2)+*(nn+2)*log(p3)+*(nn+3)*log(p4)+*(nn+4)*log(p5)-log((p1+p2+p3+p4+p5)/5);

//  cout<<x<<"\t"<<sigma;
//  for(unsigned int i=0;i<=sample_bias_step_ncut;i++){
//    cout<<" "<<*(p+i);
//  };
//  cout<<" "<<ll<<" "<<lltrue<<" "<<llmod<<"\n";

  delete[] pnc;
  delete[] T;
  delete[] p;
  delete[] nn;
  delete[] samp;

  //    cout<<x<<" "<<sqrt(s)<<" "<<sigma<<" "<<ll<<" "<<llmod<<" "<<lrat<<" "<<adcval<<"\n";
  return(lrat>adcval);
};


AdequacyTest switchtojackknife(AdequacyTest adequacy_test){
  if(adequacy_test==ADtest){
    return ADtest_jackknife;
  }else if(adequacy_test==KStest){
    return KStest_jackknife;
  }else{
    cout<<"Adequacy test not known - no jackknife alternative.\n";
    return adequacy_test;
  };
};


AdequacyTest switchtogrouped(AdequacyTest adequacy_test){
  if(adequacy_test==ChiSqtest){
    return ChiSqtestGrouped;
  }else{
    cout<<"Adequacy test not known - no grouped alternative.\n";
    return adequacy_test;
  };
};


long double testatval=0.05;

long double chisqlookup(unsigned int n,long double atval);

AdequacyTest read_adequacy_test(const char*  testname,const Distribution &dist){
  char *temp=new char[strlen(testname)+1];
  strcpy(temp,testname);
  char *at=NULL;
  unsigned int i=0;
  for(;*(temp+i)!='\0'&&*(temp+i)!='@';i++);
  if(*(temp+i)=='@'){
    at=temp+i+1;
    *(temp+i)='\0';
  };
  if(!strcmp(temp,"AD")){//Anderson-Darling test    
    if(at!=NULL){
      testatval=atof(at);
    };
    adcval=dist.get_critval(testatval,CalculateADstat);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    delete[] temp;
    return ADtest;
  }else if(!strcmp(temp,"AD_jackknife")){//Anderson-Darling test jackknife sample
    if(at!=NULL){
      testatval=atof(at);
    };
    adcval=dist.get_critval(testatval,CalculateADstat);
    cout<<"Critical value for AD test is "<<adcval<<".\n";
    delete[] temp;
    return ADtest_jackknife;
  }else if(!strcmp(temp,"KS")){//Kolmogorov-Smirnov test
    if(at!=NULL){
      testatval=atof(at);
    };
    kscval=dist.get_critval(testatval,CalculateKSstat);
    cout<<"Critical value for KS test is "<<kscval<<".\n";
    delete[] temp;
    return KStest;
  }else if(!strcmp(temp,"ChiSquare")){//Pearson's Chi-square test
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    //    cout<<"testatval="<<testatval<<"\n";
    if(grouped_data){//too early to do this.
      chisqcval=testatval;
      return ChiSqtestGrouped;
    }else{
      chisqcval=dist.get_critval(testatval,CalculateChiSqstat);
      //      cout<<"Critical value for chi-square test is "<<chisqcval<<".\n";
      return ChiSqtest;
    };
  }else if(!strcmp(temp,"LR_sample_bias")){//Likelihood ratio test vs sample bias 
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    adcval=chisqlookup(sample_bias_step_ncut,testatval);
    cout<<"Critical value for likelihood ratio test is "<<adcval<<".\n";
    return LRtestSampleBiasNormal;
  }else{
    delete[] temp;
    return NULL;
  };
};
